""" MNIST Image Classification via MLP (Supervised Learning)
    Using Tensorflow 2.0 Keras Deep Learning Framework
    Created on Sat Oct 26 2019
    @author: Jiunn-Lin Wu
"""

from __future__ import absolute_import, division, print_function, unicode_literals
import tensorflow as tf
import matplotlib.pyplot as plt
from sklearn.metrics import confusion_matrix

# Set Parameters 
num_classes = 10
img_width, img_height = 28, 28


#load MNIST data
mnist = tf.keras.datasets.mnist
(x_train, y_train), (x_test, y_test) = mnist.load_data()

#draw training images
plt.figure(figsize=(14,10))
x, y = 10, 4
for i in range(40):  
    plt.subplot(y, x, i+1)
    plt.imshow(x_train[i].reshape((img_width, img_height)),interpolation='nearest', cmap='gray')
    plt.xticks([]) 
    plt.yticks([])
plt.show()

#normalization
x_train, x_test = x_train / 255.0, x_test / 255.0

#create MLP model
model = tf.keras.models.Sequential([
  tf.keras.layers.Flatten(input_shape=(img_width, img_height)),
  tf.keras.layers.Dense(256, activation='relu'),
  tf.keras.layers.Dropout(0.3),
  tf.keras.layers.Dense(128, activation='relu'),
  tf.keras.layers.Dropout(0.2),
  tf.keras.layers.Dense(10, activation='softmax')
])

model.compile(optimizer='adam',
              loss='sparse_categorical_crossentropy',
              metrics=['accuracy'])

model.summary()

#train model
history = model.fit(x_train, y_train, epochs = 10, batch_size = 128, validation_split = 0.2)

model.save_weights('d:\mnist_tf2_mlp_.h5', save_format='h5')
#model.load_weights('d:\mnist_tf2_mlp_.h5')

#plot history 
plt.subplot(2, 1, 1)
plt.plot(history.history['loss'])
plt.plot(history.history['val_loss'])
plt.ylabel('loss')
plt.xlabel('epoch')
plt.legend(['train', 'test'], loc='upper left') 
plt.show()
plt.subplot(2, 1, 2)
plt.plot(history.history['accuracy'])
plt.plot(history.history['val_accuracy'])
plt.title('Model accuracy')
plt.ylabel('accuracy')
plt.xlabel('epoch')
plt.legend(['train', 'test'], loc='upper left') 
plt.show()

#evaluate model
loss, accuracy=model.evaluate(x_test,  y_test, verbose=2)
print(accuracy)

#prediction and compution matrix
predicted_class = model.predict_classes(x_test)
cm = confusion_matrix(y_test, predicted_class)

#draw test images 
plt.figure(figsize=(14,10))
x, y = 10, 4
for i in range(40):  
    plt.subplot(y, x, i+1)
    plt.imshow(x_test[i].reshape((img_width, img_height)),interpolation='nearest', cmap='gray')
    plt.xlabel(predicted_class[i])
    plt.xticks([])
    plt.yticks([])
plt.show()

#confusion matrix 
cm = confusion_matrix(y_test, predicted_class )

#plot confusion matrix 
import pylab as pl
pl.matshow(cm)
pl.title('Confusion matrix')
pl.colorbar()
pl.ylabel('True label')
pl.xlabel('Predicted label')
pl.show()

#draw incorrect images
wrong =[]
for i in range(10000):
    if (predicted_class[i]!=y_test[i]):
        wrong.append(i)

plt.figure(figsize=(10,7))
x, y = 8, 5
for i in range(40):  
    plt.subplot(y, x, i+1)
    plt.imshow(x_test[wrong[i]].reshape((img_width, img_height)),interpolation='nearest', cmap='gray')
    plt.xlabel(predicted_class[wrong[i]])
    plt.ylabel(y_test[wrong[i]])
    plt.xticks([])
    plt.yticks([])
plt.show() 